@@ -3,7 +3,7 @@
  * Turba directory driver implementation for PHP's PEAR database abstraction
  * layer.
  *
- * $Horde: turba/lib/Driver/sql.php,v 1.59.10.17.2.1 2008/02/15 16:44:11 chuck Exp $
+ * $Horde: turba/lib/Driver/sql.php,v 1.59.10.31 2009/07/10 00:37:32 mrubinsk Exp $
  *
  * @author  Jon Parise <jon@csh.rit.edu>
  * @package Turba
@@ -27,26 +27,62 @@ class Turba_Driver_sql extends Turba_Driver {
      */
     var $_db;
 
+    /**
+     * Handle for the current database connection, used for writing. Defaults
+     * to the same handle as $_db if a separate write database is not required.
+     *
+     * @var DB
+     */
+    var $_write_db;
+
     function _init()
     {
         include_once 'DB.php';
-        $this->_db = &DB::connect($this->_params,
-                                  array('persistent' => !empty($this->_params['persistent'])));
-        if (is_a($this->_db, 'PEAR_Error')) {
-            return $this->_db;
+        $this->_write_db = &DB::connect($this->_params,
+                                        array('persistent' => !empty($this->_params['persistent']),
+                                              'ssl' => !empty($this->_params['ssl'])));
+        if (is_a($this->_write_db, 'PEAR_Error')) {
+            return $this->_write_db;
         }
 
         // Set DB portability options.
-        switch ($this->_db->phptype) {
+        switch ($this->_write_db->phptype) {
         case 'mssql':
-            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
+            $this->_write_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
             break;
         default:
-            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+            $this->_write_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
         }
 
         if ($this->_params['phptype'] == 'oci8') {
-            $this->_db->query('ALTER SESSION SET NLS_DATE_FORMAT = \'YYYY-MM-DD\'');
+            $this->_write_db->query('ALTER SESSION SET NLS_DATE_FORMAT = \'YYYY-MM-DD\'');
+        }
+
+        /* Check if we need to set up the read DB connection
+         * seperately. */
+        if (!empty($this->_params['splitread'])) {
+            $params = array_merge($this->_params, $this->_params['read']);
+            $this->_db = &DB::connect($params,
+                                      array('persistent' => !empty($params['persistent']),
+                                            'ssl' => !empty($params['ssl'])));
+            if (is_a($this->_db, 'PEAR_Error')) {
+                return $this->_db;
+            }
+
+            // Set DB portability options.
+            switch ($this->_db->phptype) {
+            case 'mssql':
+                $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
+                break;
+            default:
+                $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+            }
+            if ($params['phptype'] == 'oci8') {
+                $this->_db->query('ALTER SESSION SET NLS_DATE_FORMAT = \'YYYY-MM-DD\'');
+            }
+        } else {
+            /* Default to the same DB handle for reads. */
+            $this->_db =& $this->_write_db;
         }
 
         return true;
@@ -57,15 +93,11 @@ class Turba_Driver_sql extends Turba_Driver {
      *
      * @return integer  The number of contacts that the user owns.
      */
-    function countContacts()
+    function count()
     {
-        static $count;
+        static $count = array();
 
-        if ($this->usingShares) {
-            $test = $this->share->get('uid');
-        } else {
-            $test = Auth::getAuth();
-        }
+        $test = $this->getContactOwner();
         if (!isset($count[$test])) {
             /* Build up the full query. */
             $query = 'SELECT COUNT(*) FROM ' . $this->_params['table'] .
@@ -73,7 +105,7 @@ class Turba_Driver_sql extends Turba_Driver {
             $values = array($test);
 
             /* Log the query at a DEBUG log level. */
-            Horde::logMessage('SQL query by Turba_Driver_sql::countContacts(): ' . $query,
+            Horde::logMessage('SQL query by Turba_Driver_sql::count(): ' . $query,
                               __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
             /* Run query. */
@@ -88,12 +120,15 @@ class Turba_Driver_sql extends Turba_Driver {
      * filtered list of results. If the criteria parameter is an empty array,
      * all records will be returned.
      *
-     * @param array $criteria  Array containing the search criteria.
-     * @param array $fields    List of fields to return.
+     * @param array $criteria      Array containing the search criteria.
+     * @param array $fields        List of fields to return.
+     * @param array $appendWhere   An additional where clause to append.
+     *                             Array should contain 'sql' and 'params'
+     *                             params are used as bind parameters.
      *
      * @return array  Hash containing the search results.
      */
-    function _search($criteria, $fields)
+    function _search($criteria, $fields, $appendWhere = array())
     {
         /* Build the WHERE clause. */
         $where = '';
@@ -116,6 +151,13 @@ class Turba_Driver_sql extends Turba_Driver {
             if (!empty($this->_params['filter'])) {
                 $where .= $this->_params['filter'];
             }
+            if (count($appendWhere)) {
+                $where .= ' AND ' . $appendWhere['sql'];
+                $values = array_merge($values, $appendWhere['params']);
+            }
+        } elseif (count($appendWhere)) {
+            $where = ' WHERE ' . $appendWhere['sql'];
+            $values = array_merge($values, $appendWhere['params']);
         }
 
         /* Build up the full query. */
@@ -154,40 +196,36 @@ class Turba_Driver_sql extends Turba_Driver {
     }
 
     /**
-     * Reads the given data from the SQL database and returns the result's
-     * fields.
+     * Reads the given data from the SQL database and returns the
+     * results.
      *
-     * @param array $criteria  Search criteria.
-     * @param string $id       Data identifier.
-     * @param array $fields    List of fields to return.
+     * @param string $key    The primary key field to use.
+     * @param mixed $ids     The ids of the contacts to load.
+     * @param string $owner  Only return contacts owned by this user.
+     * @param array $fields  List of fields to return.
      *
-     * @return  Hash containing the search results.
+     * @return array  Hash containing the search results.
      */
-    function _read($criteria, $id, $fields)
+    function _read($key, $ids, $owner, $fields, $blob_fields = array())
     {
         $values = array();
 
         $in = '';
-        if (is_array($id)) {
-            if (!count($id)) {
+        if (is_array($ids)) {
+            if (!count($ids)) {
                 return array();
             }
 
-            foreach ($id as $key) {
+            foreach ($ids as $id) {
                 $in .= empty($in) ? '?' : ', ?';
-                $values[] = $this->_convertToDriver($key);
+                $values[] = $this->_convertToDriver($id);
             }
-            $where = $criteria . ' IN (' . $in . ')';
+            $where = $key . ' IN (' . $in . ')';
         } else {
-            $where = $criteria . ' = ?';
-            $values[] = $this->_convertToDriver($id);
+            $where = $key . ' = ?';
+            $values[] = $this->_convertToDriver($ids);
         }
         if (isset($this->map['__owner'])) {
-            if ($this->usingShares) {
-                $owner = $this->share->get('uid');
-            } else {
-                $owner = Auth::getAuth();
-            }
             $where .= ' AND ' . $this->map['__owner'] . ' = ?';
             $values[] = $this->_convertToDriver($owner);
         }
@@ -195,8 +233,8 @@ class Turba_Driver_sql extends Turba_Driver {
             $where .= ' AND ' . $this->_params['filter'];
         }
 
-        $query  = 'SELECT ' . implode(', ', $fields) . ' ';
-        $query .= 'FROM ' . $this->_params['table'] . ' WHERE ' . $where;
+        $query  = 'SELECT ' . implode(', ', $fields) . ' FROM '
+            . $this->_params['table'] . ' WHERE ' . $where;
 
         /* Log the query at a DEBUG log level. */
         Horde::logMessage('SQL query by Turba_Driver_sql::_read(): ' . $query,
@@ -210,15 +248,25 @@ class Turba_Driver_sql extends Turba_Driver {
 
         $results = array();
         $iMax = count($fields);
-        if (!is_a($result, 'PEAR_Error')) {
-            foreach ($result as $row) {
-                $entry = array();
-                for ($i=0; $i < $iMax; $i++) {
-                    $field = $fields[$i];
+        foreach ($result as $row) {
+            $entry = array();
+            for ($i = 0; $i < $iMax; $i++) {
+                $field = $fields[$i];
+                if (isset($blob_fields[$field])) {
+                    switch ($this->_db->dbsyntax) {
+                    case 'pgsql':
+                    case 'mssql':
+                        $entry[$field] = pack('H' . strlen($row[$i]), $row[$i]);
+                        break;
+                    default:
+                        $entry[$field] = $row[$i];
+                        break;
+                    }
+                } else {
                     $entry[$field] = $this->_convertFromDriver($row[$i]);
                 }
-                $results[] = $entry;
             }
+            $results[] = $entry;
         }
 
         return $results;
@@ -227,19 +275,29 @@ class Turba_Driver_sql extends Turba_Driver {
     /**
      * Adds the specified object to the SQL database.
      */
-    function _add($attributes)
+    function _add($attributes, $blob_fields = array())
     {
-        $fields = array();
-        $values = array();
+        $fields = $values = array();
         foreach ($attributes as $field => $value) {
             $fields[] = $field;
-            $values[] = $this->_convertToDriver($value);
+            if (!empty($value) && isset($blob_fields[$field])) {
+                switch ($this->_write_db->dbsyntax) {
+                case 'mssql':
+                case 'pgsql':
+                    $values[] = bin2hex($value);
+                    break;
+                default:
+                    $values[] = $value;
+                }
+            } else {
+                $values[] = $this->_convertToDriver($value);
+            }
         }
+        $query  = 'INSERT INTO ' . $this->_params['table']
+            . ' (' . implode(', ', $fields) . ')'
+            . ' VALUES (' . str_repeat('?, ', count($values) - 1) . '?)';
 
-        $query  = 'INSERT INTO ' . $this->_params['table'] . ' (' . implode(', ', $fields) . ')';
-        $query .= ' VALUES (' . str_repeat('?, ', count($values) - 1) . '?)';
-
-        $result = $this->_db->query($query, $values);
+        $result = $this->_write_db->query($query, $values);
         if (is_a($result, 'PEAR_Error')) {
             Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
             return $result;
@@ -261,7 +319,7 @@ class Turba_Driver_sql extends Turba_Driver {
         Horde::logMessage('SQL query by Turba_Driver_sql::_delete(): ' . $query,
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
-        $result = $this->_db->query($query, $values);
+        $result = $this->_write_db->query($query, $values);
         if (is_a($result, 'PEAR_Error')) {
             Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
             return $result;
@@ -273,13 +331,9 @@ class Turba_Driver_sql extends Turba_Driver {
     /**
      * Deletes all contacts from a specific address book.
      *
-     * @param string  $sourceName  The owner_id of the address book to
-     *                             delete.  If omitted, the user's default
-     *                             address book will be cleared.
-     *
      * @return boolean  True if the operation worked.
      */
-    function _deleteAll($sourceName = '')
+    function _deleteAll($sourceName = null)
     {
         if (!Auth::getAuth()) {
             return PEAR::raiseError('permission denied');
@@ -287,16 +341,17 @@ class Turba_Driver_sql extends Turba_Driver {
 
         $query = 'DELETE FROM ' . $this->_params['table'] . ' WHERE owner_id = ?';
 
-        if (!$sourceName) {
+        if (empty($sourceName)) {
             $values = array(Auth::getAuth());
         } else {
             $values = array($sourceName);
         }
+
         /* Log the query at a DEBUG log level. */
         Horde::logMessage('SQL query by Turba_Driver_sql::_deleteAll(): ' . $query,
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
-        return $this->_db->query($query, $values);
+        return $this->_write_db->query($query, $values);
     }
 
     /**
@@ -304,16 +359,26 @@ class Turba_Driver_sql extends Turba_Driver {
      *
      * @return string  The object id, possibly updated.
      */
-    function _save($object_key, $object_id, $attributes)
+    function _save($object_key, $object_id, $attributes, $blob_fields = array())
     {
         $where = $object_key . ' = ?';
         unset($attributes[$object_key]);
 
-        $fields = array();
-        $values = array();
+        $fields = $values =  array();
         foreach ($attributes as $field => $value) {
             $fields[] = $field . ' = ?';
-            $values[] = $this->_convertToDriver($value);
+            if (!empty($value) && isset($blob_fields[$field])) {
+                switch ($this->_write_db->dbsyntax) {
+                case 'mssql':
+                case 'pgsql':
+                    $values[] = bin2hex($value);
+                    break;
+                default:
+                    $values[] = $value;
+                }
+            } else {
+                $values[] = $this->_convertToDriver($value);
+            }
         }
 
         $values[] = $object_id;
@@ -325,7 +390,7 @@ class Turba_Driver_sql extends Turba_Driver {
         Horde::logMessage('SQL query by Turba_Driver_sql::_save(): ' . $query,
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
-        $result = $this->_db->query($query, $values);
+        $result = $this->_write_db->query($query, $values);
         if (is_a($result, 'PEAR_Error')) {
             Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
             return $result;
@@ -377,7 +442,7 @@ class Turba_Driver_sql extends Turba_Driver {
                         $clause .= ' ' . $glue . ' ';
                     }
                     $rhs = $this->_convertToDriver($vals['test']);
-                    $binds = Horde_SQL::buildClause($this->_db, $vals['field'], $vals['op'], $rhs, true);
+                    $binds = Horde_SQL::buildClause($this->_db, $vals['field'], $vals['op'], $rhs, true, array('begin' => !empty($vals['begin'])));
                     if (is_array($binds)) {
                         $clause .= $binds[0];
                         $values = array_merge($values, $binds[1]);
@@ -398,12 +463,16 @@ class Turba_Driver_sql extends Turba_Driver {
                                 $clause .= ' ' . $key . ' ';
                             }
                             $rhs = $this->_convertToDriver($test['test']);
-                            $binds = Horde_SQL::buildClause($this->_db, $test['field'], $test['op'], $rhs, true);
-                            if (is_array($binds)) {
-                                $clause .= $binds[0];
-                                $values = array_merge($values, $binds[1]);
+                            if ($rhs == '' && $test['op'] == '=') {
+                                $clause .= '(' . Horde_SQL::buildClause($this->_db, $test['field'], '=', $rhs) . ' OR ' . $test['field'] . ' IS NULL)';
                             } else {
-                                $clause .= $binds;
+                                $binds = Horde_SQL::buildClause($this->_db, $test['field'], $test['op'], $rhs, true, array('begin' => !empty($test['begin'])));
+                                if (is_array($binds)) {
+                                    $clause .= $binds[0];
+                                    $values = array_merge($values, $binds[1]);
+                                } else {
+                                    $clause .= $binds;
+                                }
                             }
                         }
                     }
@@ -438,4 +507,93 @@ class Turba_Driver_sql extends Turba_Driver {
         return String::convertCharset($value, NLS::getCharset(), $this->_params['charset']);
     }
 
+    /**
+     * Remove all entries owned by the specified user.
+     *
+     * @param string $user  The user's data to remove.
+     *
+     * @return mixed True | PEAR_Error
+     */
+    function removeUserData($user)
+    {
+        // Make sure we are being called by an admin.
+        if (!Auth::isAdmin()) {
+            return PEAR::raiseError(_("Permission denied"));
+        }
+
+        return $this->_deleteAll($user);
+    }
+
+    /**
+     * Obtain Turba_List of items to get TimeObjects out of.
+     *
+     * @param Horde_Date $start  The starting date.
+     * @param Horde_Date $end    The ending date.
+     * @param string $field      The address book field containing the
+     *                           timeObject information (birthday, anniversary)
+     *
+     * @return mixed  A Tubra_List of objects || PEAR_Error
+     */
+    function _getTimeObjectTurbaList($start, $end, $field)
+    {
+        $t_object = $this->toDriver($field);
+        $criteria = $this->makesearch(
+            array('__owner' => $this->getContactOwner()),
+            'AND',
+            array($this->toDriver('__owner') => true),
+            false);
+
+        // Limit to entries that actually contain a birthday and that are in the
+        // date range we are looking for.
+        $criteria['AND'][] = array('field' => $t_object,
+                                   'op' => '<>',
+                                   'test' => '');
+
+        if ($start->year == $end->year) {
+            $start = sprintf('%02d-%02d', $start->month, $start->mday);
+            $end = sprintf('%02d-%02d', $end->month, $end->mday);
+            $where = array('sql' => $t_object . ' IS NOT NULL AND SUBSTR('
+                           . $t_object . ', 6, 5) BETWEEN ? AND ?',
+                           'params' => array($start, $end));
+        } else {
+            $months = array();
+            $diff = ($end->month + 12) - $start->month;
+            $newDate = new Horde_Date(array('month' => $start->month,
+                                            'mday' => $start->mday,
+                                            'year' => $start->year));
+            for ($i = 0; $i <= $diff; $i++) {
+                $months[] = sprintf('%02d', $newDate->month);
+                $newDate->month++;
+                $newDate->correct();
+            }
+            $where = array('sql' => $t_object . ' IS NOT NULL AND SUBSTR('
+                           . $t_object . ', 6, 2) IN ('
+                           . str_repeat('?,', count($months) - 1) . '?)',
+                           'params' => $months);
+        }
+
+        $fields_pre = array('__key', '__type', '__owner', 'name',
+                            'birthday', 'category', 'anniversary');
+        $fields = array();
+        foreach ($fields_pre as $field) {
+            $result = $this->toDriver($field);
+            if (is_array($result)) {
+                foreach ($result as $composite_field) {
+                    $composite_result = $this->toDriver($composite_field);
+                    if ($composite_result) {
+                        $fields[] = $composite_result;
+                    }
+                }
+            } elseif ($result) {
+                $fields[] = $result;
+            }
+        }
+
+        $res = $this->_search($criteria, $fields, $where);
+        if (is_a($res, 'PEAR_Error')) {
+            return $res;
+        }
+        return $this->_toTurbaObjects($res);
+    }
+
 }
