@@ -2,7 +2,7 @@
 /**
  * Turba external API interface.
  *
- * $Horde: turba/lib/api.php,v 1.120.2.61 2008/09/18 17:25:41 mrubinsk Exp $
+ * $Horde: turba/lib/api.php,v 1.120.2.68 2009/09/04 10:38:38 jan Exp $
  *
  * This file defines Turba's external API interface. Other applications can
  * interact with Turba through this API.
@@ -50,12 +50,12 @@ $_services['list'] = array(
 );
 
 $_services['listBy'] = array(
-    'args' => array('action' => 'string', 'timestamp' => 'int'),
+    'args' => array('action' => 'string', 'timestamp' => 'int', 'sources' => '{urn:horde}stringArray'),
     'type' => '{urn:horde}stringArray',
 );
 
 $_services['getActionTimestamp'] = array(
-    'args' => array('uid' => 'string', 'timestamp' => 'int'),
+    'args' => array('uid' => 'string', 'action' => 'string', 'sources' => '{urn:horde}stringArray'),
     'type' => 'int',
 );
 
@@ -188,6 +188,11 @@ $_services['hasComments'] = array(
     'type' => 'boolean'
 );
 
+$_services['getDefaultShare'] = array(
+    'args' => array(),
+    'type' => 'string'
+);
+
 /**
  * Removes user data.
  *
@@ -225,7 +230,7 @@ function _turba_removeUserData($user)
     }
 
     /* Only attempt share removal if we have shares configured */
-    if ($_SESSION['turba']['has_share']) {
+    if (!empty($_SESSION['turba']['has_share'])) {
         $shares = &$GLOBALS['turba_shares']->listShares(
             $user, PERMS_EDIT, $user);
 
@@ -243,7 +248,7 @@ function _turba_removeUserData($user)
                 }
             }
         }
-        
+
         /* Get a list of all shares this user has perms to and remove the perms */
         $shares = $GLOBALS['turba_shares']->listShares($user);
         if (is_a($shares, 'PEAR_Error')) {
@@ -383,6 +388,42 @@ function _turba_fields($source = null)
 
 /**
  * Browses through Turba's object tree.
+ * Retrieve the UID for the current user's default Turba share.
+ *
+ */
+function _turba_getDefaultShare()
+{
+    global $prefs;
+
+    // Bring in turba's base and a clean copy of sources.
+    require_once dirname(__FILE__) . '/base.php';
+    require TURBA_BASE . '/config/sources.php';
+
+    if (!empty($_SESSION['turba']['has_share'])) {
+        $shares = Turba::listShares(true);
+        if (is_a($shares, 'PEAR_Error')) {
+            return false;
+        }
+        foreach ($shares as $uid => $share) {
+            $params = @unserialize($share->get('params'));
+            if (empty($params['source'])) {
+                return false;
+            }
+            $driver = &Turba_Driver::factory($params['source'], $cfgSources[$params['source']]);
+            if (is_a($driver, 'PEAR_Error')) {
+                return false;
+            }
+            if ($driver->checkDefaultShare($share, $cfgSources[$params['source']])) {
+                return $uid;
+            }
+        }
+    }
+
+    // Return Turba's default_dir as default
+    return $prefs->getValue('default_dir');
+}
+
+/**
  *
  * @param string $path       The path of the tree to browse.
  * @param array $properties  The item properties to return. Defaults to 'name',
@@ -805,6 +846,7 @@ function _turba_getActionTimestamp($uid, $action, $sources = null)
         return PEAR::raiseError(_("No address book specified"), 'horde.error');
     }
 
+    $last = 0;
     foreach ($sources as $source) {
         if (empty($source) || !isset($cfgSources[$source])) {
             return PEAR::raiseError(sprintf(_("Invalid address book: %s"), $source), 'horde.error', null, null, $source);
@@ -819,12 +861,12 @@ function _turba_getActionTimestamp($uid, $action, $sources = null)
         $ts = $history->getActionTimestamp('turba:' . $driver->getName()
                                            . ':' . $uid,
                                            $action);
-        if (!empty($ts)) {
-            return $ts;
+        if (!empty($ts) && $ts > $last) {
+            $last = $ts;
         }
     }
 
-    return 0;
+    return $last;
 }
 
 /**
@@ -1429,8 +1471,13 @@ function _turba_search($names = array(), $sources = array(), $fields = array(),
                                 if (empty($value)) {
                                     continue;
                                 }
-                                $seen_key = trim(String::lower($ob->getValue('name')))
-                                    . trim(String::lower($value));
+                                if (!is_array($value)) {
+                                    $seen_key = trim(String::lower($ob->getValue('name')))
+                                        . trim(String::lower($value));
+                                } else {
+                                    $seen_key = trim(String::lower($ob->getValue('name')))
+                                        . trim(String::lower($value['load']['file']));
+                                }
                                 if (isset($attributes[$key]) &&
                                     $attributes[$key]['type'] == 'email' &&
                                     empty($seeninlist[$seen_key])) {
@@ -1591,13 +1638,22 @@ function _turba_listTimeObjectCategories()
 {
     include dirname(__FILE__) . '/../config/attributes.php';
     $categories = array();
+    include dirname(__FILE__) . '/../config/sources.php';
     foreach ($attributes as $key => $attribute) {
         if ($attribute['type'] == 'monthdayyear' &&
             !empty($attribute['time_object_label'])) {
 
-            $categories[$key] = $attribute['time_object_label'];
+            foreach ($cfgSources as $source) {
+                if (!empty($source['map'][$key])) {
+                    $categories[$key] = $attribute['time_object_label'];
+                    break;
+                }
+            }
         }
     }
+
+
+
     return $categories;
 }
 
@@ -1649,6 +1705,7 @@ function _turba_listTimeObjects($time_categories, $start, $end)
             $objects = array_merge($objects, $new_objects);
         }
     }
+
     return $objects;
 }
 
